

avl_node_attrs=['ctrl_type','activity_type','production','attraction']
avl_node_attrs_for_num=['ctrl_type','activity_type']
avl_node_attrs_for_range=['production','attraction']

avl_link_attrs=['link_type_name','allowed_uses','free_speed','lanes','capacity','length']
avl_link_attrs_for_num=['link_type_name','allowed_uses']
avl_link_attrs_for_range=['free_speed','lanes','capacity','length']

avl_poi_attrs=['building','activity_zone_id']